package com.graphaware.neo4j.utils;

import org.apache.log4j.Logger;
import org.neo4j.graphdb.Direction;
import org.neo4j.graphdb.Node;
import org.neo4j.graphdb.Relationship;

import static org.neo4j.graphdb.Direction.*;

/**
 * Utility class for static methods related to {@link Relationship} {@link Direction}.
 */
public final class DirectionUtils {
    private static final Logger LOG = Logger.getLogger(DirectionUtils.class);

    /**
     * Resolve the direction of a relationship from a node's point of view. If the start and end nodes of the relationship
     * are the same node, {@link Direction#OUTGOING} will be returned.
     *
     * @param relationship to resolve direction for.
     * @param pointOfView  direction will be from this node's point of view.
     * @return direction of the relationship from the given node's point of view.
     */
    public static Direction resolveDirection(Relationship relationship, Node pointOfView) {
        return resolveDirection(relationship, pointOfView, OUTGOING);
    }

    /**
     * Resolve the direction of a relationship from a node's point of view.
     *
     * @param relationship     to resolve direction for.
     * @param pointOfView      direction will be from this node's point of view.
     * @param defaultDirection value returned in case the relationship is a "self-relationships", i.e. the start and
     *                         end nodes are the same.
     * @return direction of the relationship from the given node's point of view.
     */
    public static Direction resolveDirection(Relationship relationship, Node pointOfView, Direction defaultDirection) {
        if (relationship.getEndNode().getId() != pointOfView.getId() && relationship.getStartNode().getId() != pointOfView.getId()) {
            String message = "Provided relationship (" + relationship.getId() + ") does not have node (" + pointOfView.getId() + ") on either of its ends!";
            LOG.error(message);
            throw new IllegalArgumentException(message);
        }

        if (relationship.getEndNode().getId() == relationship.getStartNode().getId()) {
            return defaultDirection;
        }

        return ((relationship.getStartNode().getId() == pointOfView.getId()) ? OUTGOING : INCOMING);
    }

    /**
     * Does the given relationships match the given direction from the given node's point of view? By matches, we mean
     * that either the directions are the same, or the given direction is {@link Direction#BOTH}.
     *
     * @param relationship for which to check direction match.
     * @param pointOfView  direction will be from this node's point of view.
     * @param direction    to match. If {@link Direction#BOTH}, then the method always returns <code>true</code>.
     * @return true iff the relationship matches the direction from the given node's point of view.
     */
    public static boolean matches(Relationship relationship, Node pointOfView, Direction direction) {
        Direction actualDirection = resolveDirection(relationship, pointOfView, BOTH);
        return BOTH.equals(direction) || BOTH.equals(actualDirection) || actualDirection.equals(direction);
    }

    private DirectionUtils() {
    }
}
