package com.graphaware.neo4j.utils;

import com.graphaware.neo4j.common.Change;
import org.apache.log4j.Logger;
import org.neo4j.graphdb.Node;
import org.neo4j.graphdb.PropertyContainer;
import org.neo4j.graphdb.Relationship;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import static com.graphaware.neo4j.utils.ArrayUtils.isPrimitiveOrStringArray;
import static com.graphaware.neo4j.utils.ArrayUtils.primitiveArrayToString;

/**
 * Utility methods for dealing with {@link PropertyContainer}s.
 */
public final class PropertyContainerUtils {
    private static final Logger LOG = Logger.getLogger(PropertyContainerUtils.class);

    /**
     * Convert a collection of {@link PropertyContainer}s to a map of {@link PropertyContainer}s keyed by their ID.
     *
     * @param propertyContainers to convert.
     * @param <T>                type of the {@link PropertyContainer}.
     * @return map keyed by {@link PropertyContainer} ID with the actual {@link PropertyContainer}s as values.
     */
    public static <T extends PropertyContainer> Map<Long, T> propertyContainersToMap(Collection<T> propertyContainers) {
        Map<Long, T> result = new HashMap<>();

        for (T propertyContainer : propertyContainers) {
            result.put(id(propertyContainer), propertyContainer);
        }

        return result;
    }

    /**
     * Convert a collection of {@link Change}s of {@link PropertyContainer} to a map of {@link Change}s keyed by the
     * {@link PropertyContainer} ID.
     *
     * @param changes to convert.
     * @param <T>     type of the {@link PropertyContainer}.
     * @return map keyed by {@link PropertyContainer} ID with the actual {@link Change}s as values.
     * @throws IllegalArgumentException in case the two {@link PropertyContainer}s contained in a {@link Change} do not
     *                                  have the same IDs.
     */
    public static <T extends PropertyContainer> Map<Long, Change<T>> changesToMap(Collection<Change<T>> changes) {
        Map<Long, Change<T>> result = new HashMap<>();

        for (Change<T> change : changes) {
            long id = id(change.getPrevious());
            if (id != id(change.getCurrent())) {
                throw new IllegalArgumentException("IDs of the Property Containers in Change do not match!");
            }
            result.put(id, change);
        }

        return result;
    }

    /**
     * Get ID from a {@link PropertyContainer}.
     *
     * @param propertyContainer to get ID from. Must be a {@link Node} or {@link Relationship}.
     * @return ID
     * @throws IllegalStateException in case the propertyContainer is not a {@link Node} or a {@link Relationship}.
     */
    public static long id(PropertyContainer propertyContainer) {
        if (Node.class.isAssignableFrom(propertyContainer.getClass())) {
            return ((Node) propertyContainer).getId();
        }

        if (Relationship.class.isAssignableFrom(propertyContainer.getClass())) {
            return ((Relationship) propertyContainer).getId();
        }

        throw new IllegalStateException("Unknown Property Container: " + propertyContainer.getClass().getName());
    }

    /**
     * Make sure a key is not null or empty.
     *
     * @param key the key to check.
     * @return the same key.
     * @throws IllegalArgumentException if null or empty String.
     */
    public static String cleanKey(String key) {
        if (key == null || "".equals(key.trim())) {
            throw new IllegalArgumentException("Key must not be null or empty!");
        }
        return key;
    }

    /**
     * Convert a property value to String. If the value is <code>null</code>, then it will be converted to an empty String.
     *
     * @param value to convert.
     * @return property value as String.
     */
    public static String valueToString(Object value) {
        if (value == null) {
            return "";
        }

        if (isPrimitiveOrStringArray(value)) {
            return primitiveArrayToString(value);
        }
        return String.valueOf(value);
    }

    /**
     * Convert properties from a {@link PropertyContainer} to a {@link Map} of {@link String}s, where the key is the
     * property key and value is the property value converted to {@link String}. Keys must not be <code>null</code>
     * (not allowed by Neo4j anyway) or empty (not allowed by GraphAware). <code>Null</code> values will be converted
     * to empty {@link String}s.
     *
     * @param propertyContainer to convert properties from.
     * @return converted properties.
     */
    public static Map<String, String> propertiesToMap(PropertyContainer propertyContainer) {
        Map<String, String> result = new HashMap<>();
        for (String key : propertyContainer.getPropertyKeys()) {
            result.put(cleanKey(key), valueToString(propertyContainer.getProperty(key)));
        }
        return result;
    }

    /**
     * Convert properties from a {@link Map} to a {@link Map} of "clean properties", where the key is the
     * property key and value is the property value converted to {@link String}. Keys must not be <code>null</code>
     * (not allowed by Neo4j anyway) or empty (not allowed by GraphAware). <code>Null</code> values will be converted
     * to empty {@link String}s.
     *
     * @param properties to cleanup
     * @return cleaned properties.
     */
    public static Map<String, String> cleanProperties(Map<String, String> properties) {
        Map<String, String> result = new HashMap<>();
        for (String key : properties.keySet()) {
            result.put(cleanKey(key), valueToString(properties.get(key)));
        }
        return result;
    }

    private PropertyContainerUtils() {
    }
}
